@charset "UTF-8";

:root {
    --aspect-ratio: 1.5;
}

[style*="--aspect-ratio"] {

    position: relative;
}

[style*="--aspect-ratio"]::before {
    display: block;
    content: "";
    padding-bottom: calc(100% / (var(--aspect-ratio)));
}

[style*="--aspect-ratio"] > *:first-child {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
}
